// Copyright 2022 Chaos Mesh Authors.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

package v1alpha1

import (
	"fmt"
	"reflect"

	"k8s.io/apimachinery/pkg/util/validation/field"

	"github.com/chaos-mesh/chaos-mesh/api/genericwebhook"
)

type DiskName string
type LUN int

func (in *DiskName) Validate(root interface{}, path *field.Path) field.ErrorList {
	allErrs := field.ErrorList{}

	azurechaos := root.(*AzureChaos)
	if azurechaos.Spec.Action == AzureDiskDetach {
		if in == nil {
			err := fmt.Errorf("the name of data disk should not be empty on %s action", azurechaos.Spec.Action)
			allErrs = append(allErrs, field.Invalid(path, in, err.Error()))
		}
	}

	return allErrs
}

func (in *LUN) Validate(root interface{}, path *field.Path) field.ErrorList {
	allErrs := field.ErrorList{}

	azurechaos := root.(*AzureChaos)
	if azurechaos.Spec.Action == AzureDiskDetach {
		if in == nil {
			err := fmt.Errorf("the LUN of data disk should not be empty on %s action", azurechaos.Spec.Action)
			allErrs = append(allErrs, field.Invalid(path, in, err.Error()))
		}
	}

	return allErrs
}

// Validate validates the azure chaos actions
func (in *AzureChaosAction) Validate(root interface{}, path *field.Path) field.ErrorList {
	allErrs := field.ErrorList{}

	// in cannot be nil
	switch *in {
	case AzureVmStop, AzureDiskDetach:
	case AzureVmRestart:
	default:
		err := fmt.Errorf("azurechaos have unknown action type")
		log.Error(err, "Wrong AzureChaos Action type")

		allErrs = append(allErrs, field.Invalid(path, in, err.Error()))
	}
	return allErrs
}

func init() {
	genericwebhook.Register("DiskName", reflect.PtrTo(reflect.TypeOf(DiskName(""))))
	genericwebhook.Register("LUN", reflect.PtrTo(reflect.TypeOf(LUN(0))))
}
